import os


class MediaTypes(object):

    def __init__(self, default='application/octet-stream', extra_types=None):
        self.types_map = default_types()
        self.default = default
        if extra_types:
            self.types_map.update(extra_types)

    def get_type(self, path):
        name = os.path.basename(path).lower()
        media_type = self.types_map.get(name)
        if media_type is not None:
            return media_type
        extension = os.path.splitext(name)[1]
        return self.types_map.get(extension, self.default)


def default_types():
    """
    We use our own set of default media types rather than the system-supplied
    ones. This ensures consistent media type behaviour across varied
    environments.  The defaults are based on those shipped with nginx, with
    some custom additions.
    """

    return {
        '.3gp': 'video/3gpp',
        '.3gpp': 'video/3gpp',
        '.7z': 'application/x-7z-compressed',
        '.ai': 'application/postscript',
        '.asf': 'video/x-ms-asf',
        '.asx': 'video/x-ms-asf',
        '.atom': 'application/atom+xml',
        '.avi': 'video/x-msvideo',
        '.bmp': 'image/x-ms-bmp',
        '.cco': 'application/x-cocoa',
        '.crt': 'application/x-x509-ca-cert',
        '.css': 'text/css',
        '.der': 'application/x-x509-ca-cert',
        '.doc': 'application/msword',
        '.docx': 'application/vnd.openxmlformats-officedocument.wordprocessingml.document',
        '.ear': 'application/java-archive',
        '.eot': 'application/vnd.ms-fontobject',
        '.eps': 'application/postscript',
        '.flv': 'video/x-flv',
        '.gif': 'image/gif',
        '.hqx': 'application/mac-binhex40',
        '.htc': 'text/x-component',
        '.htm': 'text/html',
        '.html': 'text/html',
        '.ico': 'image/x-icon',
        '.jad': 'text/vnd.sun.j2me.app-descriptor',
        '.jar': 'application/java-archive',
        '.jardiff': 'application/x-java-archive-diff',
        '.jng': 'image/x-jng',
        '.jnlp': 'application/x-java-jnlp-file',
        '.jpeg': 'image/jpeg',
        '.jpg': 'image/jpeg',
        '.js': 'application/javascript',
        '.json': 'application/json',
        '.kar': 'audio/midi',
        '.kml': 'application/vnd.google-earth.kml+xml',
        '.kmz': 'application/vnd.google-earth.kmz',
        '.m3u8': 'application/vnd.apple.mpegurl',
        '.m4a': 'audio/x-m4a',
        '.m4v': 'video/x-m4v',
        '.mid': 'audio/midi',
        '.midi': 'audio/midi',
        '.mml': 'text/mathml',
        '.mng': 'video/x-mng',
        '.mov': 'video/quicktime',
        '.mp3': 'audio/mpeg',
        '.mp4': 'video/mp4',
        '.mpeg': 'video/mpeg',
        '.mpg': 'video/mpeg',
        '.ogg': 'audio/ogg',
        '.pdb': 'application/x-pilot',
        '.pdf': 'application/pdf',
        '.pem': 'application/x-x509-ca-cert',
        '.pl': 'application/x-perl',
        '.pm': 'application/x-perl',
        '.png': 'image/png',
        '.ppt': 'application/vnd.ms-powerpoint',
        '.pptx': 'application/vnd.openxmlformats-officedocument.presentationml.presentation',
        '.prc': 'application/x-pilot',
        '.ps': 'application/postscript',
        '.ra': 'audio/x-realaudio',
        '.rar': 'application/x-rar-compressed',
        '.rpm': 'application/x-redhat-package-manager',
        '.rss': 'application/rss+xml',
        '.rtf': 'application/rtf',
        '.run': 'application/x-makeself',
        '.sea': 'application/x-sea',
        '.shtml': 'text/html',
        '.sit': 'application/x-stuffit',
        '.svg': 'image/svg+xml',
        '.svgz': 'image/svg+xml',
        '.swf': 'application/x-shockwave-flash',
        '.tcl': 'application/x-tcl',
        '.tif': 'image/tiff',
        '.tiff': 'image/tiff',
        '.tk': 'application/x-tcl',
        '.ts': 'video/mp2t',
        '.txt': 'text/plain',
        '.war': 'application/java-archive',
        '.wbmp': 'image/vnd.wap.wbmp',
        '.webm': 'video/webm',
        '.webp': 'image/webp',
        '.wml': 'text/vnd.wap.wml',
        '.wmlc': 'application/vnd.wap.wmlc',
        '.wmv': 'video/x-ms-wmv',
        '.woff': 'application/font-woff',
        '.woff2': 'font/woff2',
        '.xhtml': 'application/xhtml+xml',
        '.xls': 'application/vnd.ms-excel',
        '.xlsx': 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
        '.xml': 'text/xml',
        '.xpi': 'application/x-xpinstall',
        '.xspf': 'application/xspf+xml',
        '.zip': 'application/zip',
        'apple-app-site-association': 'application/pkc7-mime',
        # Adobe Products - see:
        # https://www.adobe.com/devnet-docs/acrobatetk/tools/AppSec/xdomain.html#policy-file-host-basics
        'crossdomain.xml': 'text/x-cross-domain-policy'
    }



#!/usr/bin/env python
import pprint


EXTRA_MIMETYPES = {
    'apple-app-site-association': 'application/pkc7-mime',
    '.woff': 'application/font-woff',
    '.woff2': 'font/woff2'
}


FUNCTION_TEMPLATE = """
def default_types():
    {triple_quote}
    We use our own set of default media types rather than the system-supplied
    ones. This ensures consistent media type behaviour across varied
    environments.  The defaults are based on those shipped with nginx, with
    some custom additions.
    {triple_quote}

    return {{
        {types_map}
    }}
""".lstrip()


NGINX_CONFIG_FILE = '/etc/nginx/mime.types'


def get_default_types_function():
    types_map = get_types_map()
    types_map_str = pprint.pformat(types_map, indent=8).strip('{} ')
    return FUNCTION_TEMPLATE.format(
            triple_quote='"""',
            types_map=types_map_str)


def get_types_map():
    types_map = {}
    with open(NGINX_CONFIG_FILE, 'r') as f:
        for line in f:
            line = line.strip()
            if not line.endswith(';'):
                continue
            line = line.rstrip(';')
            bits = line.split()
            media_type = bits[0]
            # This is the default media type anyway, no point specifying
            # it explicitly
            if media_type == 'application/octet-stream':
                continue
            extensions = bits[1:]
            for extension in extensions:
                types_map['.'+extension] = media_type
    types_map.update(EXTRA_MIMETYPES)
    return types_map


if __name__ == '__main__':
    print get_default_types_function()



from __future__ import unicode_literals

import os
import threading
import warnings
from wsgiref.simple_server import make_server, WSGIRequestHandler

import requests

warnings.filterwarnings(action='ignore', category=DeprecationWarning,
                        module='requests')


TEST_FILE_PATH = os.path.join(os.path.dirname(__file__), 'test_files')


class SilentWSGIHandler(WSGIRequestHandler):
    def log_message(*args):
        pass


class TestServer(object):
    """
    Wraps a WSGI application and allows you to make real HTTP
    requests against it
    """

    def __init__(self, application):
        self.application = application
        self.server = make_server('127.0.0.1', 0, application,
                                  handler_class=SilentWSGIHandler)

    def get(self, *args, **kwargs):
        return self.request('get', *args, **kwargs)

    def request(self, method, path, *args, **kwargs):
        url = 'http://{0[0]}:{0[1]}{1}'.format(self.server.server_address, path)
        thread = threading.Thread(target=self.server.handle_request)
        thread.start()
        response = requests.request(method, url, *args, **kwargs)
        thread.join()
        return response


class Files(object):
    def __init__(self, directory, **files):
        self.directory = os.path.join(TEST_FILE_PATH, directory)
        for name, path in files.items():
            url = '/' + path
            with open(os.path.join(self.directory, path), 'rb') as f:
                content = f.read()
            setattr(self, name + '_path', path)
            setattr(self, name + '_url', url)
            setattr(self, name + '_content', content)



from django.contrib.staticfiles.storage import HashedFilesMixin
from django.test import SimpleTestCase
from django.test.utils import override_settings

from whitenoise.storage import HelpfulExceptionMixin, MissingFileError


@override_settings()
class DjangoWhiteNoiseStorageTest(SimpleTestCase):

    def test_make_helpful_exception(self):
        class TriggerException(HashedFilesMixin):
            def exists(self, path):
                return False
        exception = None
        try:
            TriggerException().hashed_name('/missing/file.png')
        except ValueError as e:
            exception = e
        helpful_exception = HelpfulExceptionMixin().make_helpful_exception(
                                exception,
                                'styles/app.css'
                            )
        self.assertIsInstance(helpful_exception, MissingFileError)



from __future__ import absolute_import

import os
from posixpath import basename

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
try:
    from django.contrib.staticfiles.storage import staticfiles_storage
except ImproperlyConfigured:
    if not os.environ.get('DJANGO_SETTINGS_MODULE'):
        raise ImproperlyConfigured(
                "'DJANGO_SETTINGS_MODULE' environment variable must be set "
                "before importing 'whitenoise.django'")
    else:
        raise
from django.contrib.staticfiles import finders
from django.utils.six.moves.urllib.parse import urlparse

from .base import WhiteNoise
# Import here under an alias for backwards compatibility
from .storage import (CompressedManifestStaticFilesStorage as
                      GzipManifestStaticFilesStorage)
from .utils import (decode_if_byte_string, ensure_leading_trailing_slash,
                    IsDirectoryError)


__all__ = ['DjangoWhiteNoise', 'GzipManifestStaticFilesStorage']


class DjangoWhiteNoise(WhiteNoise):

    config_attrs = WhiteNoise.config_attrs + (
            'root', 'use_finders', 'static_prefix')
    root = None
    use_finders = False
    static_prefix = None

    def __init__(self, application, settings=settings):
        self.configure_from_settings(settings)
        self.check_settings(settings)
        super(DjangoWhiteNoise, self).__init__(application)
        if self.static_root:
            self.add_files(self.static_root, prefix=self.static_prefix)
        if self.root:
            self.add_files(self.root)

    def configure_from_settings(self, settings):
        # Default configuration
        self.charset = settings.FILE_CHARSET
        self.autorefresh = settings.DEBUG
        self.use_finders = settings.DEBUG
        self.static_prefix = urlparse(settings.STATIC_URL or '').path
        if settings.DEBUG:
            self.max_age = 0
        # Allow settings to override default attributes
        for attr in self.config_attrs:
            settings_key = 'WHITENOISE_{0}'.format(attr.upper())
            try:
                value = getattr(settings, settings_key)
            except AttributeError:
                pass
            else:
                value = decode_if_byte_string(value)
                setattr(self, attr, value)
        self.static_prefix = ensure_leading_trailing_slash(self.static_prefix)
        self.static_root = decode_if_byte_string(settings.STATIC_ROOT)

    def check_settings(self, settings):
        if self.use_finders and not self.autorefresh:
            raise ImproperlyConfigured(
                'WHITENOISE_USE_FINDERS can only be enabled in development '
                'when WHITENOISE_AUTOREFRESH is also enabled.'
            )

    def find_file(self, url):
        if self.use_finders and url.startswith(self.static_prefix):
            path = finders.find(url[len(self.static_prefix):])
            if path:
                try:
                    return self.get_static_file(path, url)
                except IsDirectoryError:
                    return None
        return super(DjangoWhiteNoise, self).find_file(url)

    def is_immutable_file(self, path, url):
        """
        Determine whether given URL represents an immutable file (i.e. a
        file with a hash of its contents as part of its name) which can
        therefore be cached forever
        """
        if not url.startswith(self.static_prefix):
            return False
        name = url[len(self.static_prefix):]
        name_without_hash = self.get_name_without_hash(name)
        if name == name_without_hash:
            return False
        static_url = self.get_static_url(name_without_hash)
        # If the static_url function maps the name without hash
        # back to the original name, then we know we've got a
        # versioned filename
        if static_url and basename(static_url) == basename(url):
            return True
        return False

    def get_name_without_hash(self, filename):
        """
        Removes the version hash from a filename e.g, transforms
        'css/application.f3ea4bcc2.css' into 'css/application.css'

        Note: this is specific to the naming scheme used by Django's
        CachedStaticFilesStorage. You may have to override this if
        you are using a different static files versioning system
        """
        name_with_hash, ext = os.path.splitext(filename)
        name = os.path.splitext(name_with_hash)[0]
        return name + ext

    def get_static_url(self, name):
        try:
            return decode_if_byte_string(staticfiles_storage.url(name))
        except ValueError:
            return None



"""
Very partial backport of the `http.HTTPStatus` enum from Python 3.5

This implements just enough of the interface for our purposes, it does not
attempt to be a full implementation.
"""


class HTTPStatus(int):

    phrase = None

    def __new__(cls, code, phrase):
        instance = int.__new__(cls, code)
        instance.phrase = phrase
        return instance


HTTPStatus.OK = HTTPStatus(200, 'OK')
HTTPStatus.NOT_MODIFIED = HTTPStatus(304, 'Not Modified')
HTTPStatus.METHOD_NOT_ALLOWED = HTTPStatus(405, 'Method Not Allowed')



from django.conf.urls import url
from django.http import HttpResponse


def hello_world(reqeust):
    return HttpResponse(content='Hello Word', content_type='text/plain')

urlpatterns = [
    url(r'^hello$', hello_world),
]



from __future__ import absolute_import

import os
import re
import textwrap

from django.conf import settings
from django.contrib.staticfiles.storage import ManifestStaticFilesStorage

from .compress import Compressor


class CompressedStaticFilesMixin(object):
    """
    Wraps a StaticFilesStorage instance to create compressed versions of its
    output files
    """

    def post_process(self, *args, **kwargs):
        files = super(CompressedStaticFilesMixin, self).post_process(*args, **kwargs)
        if not kwargs.get('dry_run'):
            files = self.post_process_with_compression(files)
        return files

    def post_process_with_compression(self, files):
        extensions = getattr(settings,
                             'WHITENOISE_SKIP_COMPRESS_EXTENSIONS', None)
        compressor = Compressor(extensions=extensions, quiet=True)
        for name, hashed_name, processed in files:
            if self.should_compress(compressor, name, processed):
                compressor.compress(self.path(name))
                if hashed_name is not None:
                    compressor.compress(self.path(hashed_name))
            yield name, hashed_name, processed

    def should_compress(self, compressor, name, processed):
        if isinstance(processed, Exception):
            return False
        else:
            return compressor.should_compress(name)


class HelpfulExceptionMixin(object):
    """
    If a CSS file contains references to images, fonts etc that can't be found
    then Django's `post_process` blows up with a not particularly helpful
    ValueError that leads people to think WhiteNoise is broken.

    Here we attempt to intercept such errors and reformat them to be more
    helpful in revealing the source of the problem.
    """

    ERROR_MSG_RE = re.compile("^The file '(.+)' could not be found")

    ERROR_MSG = textwrap.dedent(u"""\
        {orig_message}

        The {ext} file '{filename}' references a file which could not be found:
          {missing}

        Please check the URL references in this {ext} file, particularly any
        relative paths which might be pointing to the wrong location.
        """)

    def post_process(self, *args, **kwargs):
        files = super(HelpfulExceptionMixin, self).post_process(*args, **kwargs)
        for name, hashed_name, processed in files:
            if isinstance(processed, Exception):
                processed = self.make_helpful_exception(processed, name)
            yield name, hashed_name, processed

    def make_helpful_exception(self, exception, name):
        if isinstance(exception, ValueError):
            message = exception.args[0] if len(exception.args) else ''
            # Stringly typed exceptions. Yay!
            match = self.ERROR_MSG_RE.search(message)
            if match:
                extension = os.path.splitext(name)[1].lstrip('.').upper()
                message = self.ERROR_MSG.format(
                        orig_message=message,
                        filename=name,
                        missing=match.group(1),
                        ext=extension)
                exception = MissingFileError(message)
        return exception


class MissingFileError(ValueError):
    pass


class CompressedManifestStaticFilesStorage(
        HelpfulExceptionMixin, CompressedStaticFilesMixin,
        ManifestStaticFilesStorage):
    pass



from __future__ import absolute_import

from django.http import FileResponse

from whitenoise.django import DjangoWhiteNoise


class WhiteNoiseMiddleware(DjangoWhiteNoise):
    """
    Wrap DjangoWhiteNoise to allow it to function as Django middleware, rather
    than WSGI middleware

    This functions as both old- and new-style middleware, so can be included in
    either MIDDLEWARE or MIDDLEWARE_CLASSES.
    """

    def __init__(self, get_response=None):
        self.get_response = get_response
        # We pass None for `application`
        super(WhiteNoiseMiddleware, self).__init__(None)

    def __call__(self, request):
        response = self.process_request(request)
        if response is None:
            response = self.get_response(request)
        return response

    def process_request(self, request):
        if self.autorefresh:
            static_file = self.find_file(request.path_info)
        else:
            static_file = self.files.get(request.path_info)
        if static_file is not None:
            return self.serve(static_file, request)

    def serve(self, static_file, request):
        response = static_file.get_response(request.method, request.META)
        status = int(response.status)
        http_response = FileResponse(response.file or (), status=status)
        # Remove default content-type
        del http_response['content-type']
        for key, value in response.headers:
            http_response[key] = value
        return http_response



"""
This file exists solely to shadow the `runserver` command provided by
`django.contrib.staticfiles` and restore the original `runserver` behaviour
"""
from django.core.management.commands.runserver import Command

# Keep flake8 happy
__all__ = ['Command']



from collections import namedtuple
from email.utils import parsedate
try:
    from http import HTTPStatus
except ImportError:
    from .httpstatus_backport import HTTPStatus
import re
from wsgiref.headers import Headers

from .utils import MissingFileError, stat_regular_file


Response = namedtuple('Response', ('status', 'headers', 'file'))

NOT_ALLOWED = Response(HTTPStatus.METHOD_NOT_ALLOWED,
                       (('Allow', 'GET, HEAD'),),
                       None)

ACCEPT_GZIP_RE = re.compile(r'\bgzip\b')
ACCEPT_BROTLI_RE = re.compile(r'\bbr\b')

# Headers which should be returned with a 304 Not Modified response as
# specified here: http://tools.ietf.org/html/rfc7232#section-4.1
NOT_MODIFIED_HEADERS = ('Cache-Control', 'Content-Location', 'Date', 'ETag',
                        'Expires', 'Vary')
NOT_MODIFIED_HEADER_RE = re.compile('^({})$'.format(
        '|'.join(map(re.escape, NOT_MODIFIED_HEADERS))), re.IGNORECASE)


class StaticFile(object):

    def __init__(self, path, headers):
        plain_file, gzip_file, brotli_file = get_alternatives(path, headers)
        self.plain_file = file_tuple(plain_file)
        self.gzip_file = file_tuple(gzip_file)
        self.brotli_file = file_tuple(brotli_file)
        self.last_modified = parsedate(headers['Last-Modified'])
        self.not_modified_response = get_not_modified_response(self.plain_file[1])

    def get_response(self, method, request_headers):
        if method != 'GET' and method != 'HEAD':
            return NOT_ALLOWED
        elif self.file_not_modified(request_headers):
            return self.not_modified_response
        path, headers = self.get_path_and_headers(request_headers)
        if method != 'HEAD':
            file_handle = open(path, 'rb')
        else:
            file_handle = None
        return Response(HTTPStatus.OK, headers, file_handle)

    def get_path_and_headers(self, request_headers):
        accept_encoding = request_headers.get('HTTP_ACCEPT_ENCODING', '')
        if self.brotli_file and ACCEPT_BROTLI_RE.search(accept_encoding):
            return self.brotli_file
        if self.gzip_file and ACCEPT_GZIP_RE.search(accept_encoding):
            return self.gzip_file
        return self.plain_file

    def file_not_modified(self, request_headers):
        try:
            last_requested = request_headers['HTTP_IF_MODIFIED_SINCE']
        except KeyError:
            return False
        return parsedate(last_requested) >= self.last_modified


def get_alternatives(path, headers):
    gzip_file = get_alternative_encoding(path, headers, '.gz', 'gzip')
    brotli_file = get_alternative_encoding(path, headers, '.br', 'br')
    if gzip_file or brotli_file:
        headers['Vary'] = 'Accept-Encoding'
    plain_file = (path, headers)
    return plain_file, gzip_file, brotli_file


def get_alternative_encoding(path, headers, suffix, encoding):
    alt_path = path + suffix
    try:
        alt_size = stat_regular_file(alt_path).st_size
    except MissingFileError:
        return None
    alt_headers = Headers(headers.items())
    alt_headers['Vary'] = 'Accept-Encoding'
    alt_headers['Content-Encoding'] = encoding
    alt_headers['Content-Length'] = str(alt_size)
    return alt_path, alt_headers


def file_tuple(path_and_headers):
    if path_and_headers is None:
        return None
    else:
        path, headers = path_and_headers
        return path, tuple(headers.items())


def get_not_modified_response(headers):
    not_modified_headers = tuple([
        item for item in headers
        if NOT_MODIFIED_HEADER_RE.match(item[0])])
    return Response(HTTPStatus.NOT_MODIFIED,  not_modified_headers, None)



from __future__ import unicode_literals

import shutil
import tempfile

import django
from django.test import SimpleTestCase
from django.test.utils import override_settings
from django.conf import settings
from django.contrib.staticfiles import storage, finders
from django.core.wsgi import get_wsgi_application
from django.core.management import call_command
from django.utils.functional import empty

from .utils import TestServer, Files

from whitenoise.django import DjangoWhiteNoise

django.setup()


def reset_lazy_object(obj):
    obj._wrapped = empty


@override_settings()
class DjangoWhiteNoiseTest(SimpleTestCase):

    @classmethod
    def setUpClass(cls):
        cls.static_files = Files('static', css='styles.css', nonascii='nonascii\u2713.txt')
        cls.root_files = Files('root', robots='robots.txt')
        cls.tmp = tempfile.mkdtemp()
        settings.STATICFILES_STORAGE = 'whitenoise.storage.CompressedManifestStaticFilesStorage'
        settings.STATICFILES_DIRS = [cls.static_files.directory]
        settings.STATIC_ROOT = cls.tmp
        settings.WHITENOISE_ROOT = cls.root_files.directory
        # Collect static files into STATIC_ROOT
        call_command('collectstatic', verbosity=0, interactive=False)
        # Initialize test application
        cls.application = cls.init_application()
        cls.server = TestServer(cls.application)
        super(DjangoWhiteNoiseTest, cls).setUpClass()

    @classmethod
    def init_application(cls):
        django_app = get_wsgi_application()
        return DjangoWhiteNoise(django_app)

    @classmethod
    def tearDownClass(cls):
        super(DjangoWhiteNoiseTest, cls).tearDownClass()
        reset_lazy_object(storage.staticfiles_storage)
        # Remove temporary directory
        shutil.rmtree(cls.tmp)

    def test_get_root_file(self):
        response = self.server.get(self.root_files.robots_url)
        self.assertEqual(response.content, self.root_files.robots_content)

    def test_versioned_file_cached_forever(self):
        url = storage.staticfiles_storage.url(self.static_files.css_path)
        response = self.server.get(url)
        self.assertEqual(response.content, self.static_files.css_content)
        self.assertEqual(response.headers.get('Cache-Control'),
                         'max-age={}, public, immutable'.format(DjangoWhiteNoise.FOREVER))

    def test_unversioned_file_not_cached_forever(self):
        url = settings.STATIC_URL + self.static_files.css_path
        response = self.server.get(url)
        self.assertEqual(response.content, self.static_files.css_content)
        self.assertEqual(response.headers.get('Cache-Control'),
                         'max-age={}, public'.format(DjangoWhiteNoise.max_age))

    def test_get_gzip(self):
        url = storage.staticfiles_storage.url(self.static_files.css_path)
        response = self.server.get(url)
        self.assertEqual(response.content, self.static_files.css_content)
        self.assertEqual(response.headers['Content-Encoding'], 'gzip')
        self.assertEqual(response.headers['Vary'], 'Accept-Encoding')

    def test_no_content_type_when_not_modified(self):
        last_mod = 'Fri, 11 Apr 2100 11:47:06 GMT'
        url = settings.STATIC_URL + self.static_files.css_path
        response = self.server.get(url, headers={'If-Modified-Since': last_mod})
        self.assertNotIn('Content-Type', response.headers)

    def test_get_nonascii_file(self):
        url = settings.STATIC_URL + self.static_files.nonascii_path
        response = self.server.get(url)
        self.assertEqual(response.content, self.static_files.nonascii_content)


@override_settings()
class UseFindersTest(SimpleTestCase):

    @classmethod
    def setUpClass(cls):
        cls.static_files = Files('static', css='styles.css')
        settings.STATICFILES_DIRS = [cls.static_files.directory]
        settings.WHITENOISE_USE_FINDERS = True
        settings.WHITENOISE_AUTOREFRESH = True
        # Clear cache to pick up new settings
        try:
            finders.get_finder.cache_clear()
        except AttributeError:
            finders._finders.clear()
        # Initialize test application
        cls.application = cls.init_application()
        cls.server = TestServer(cls.application)
        super(UseFindersTest, cls).setUpClass()

    @classmethod
    def init_application(cls):
        django_app = get_wsgi_application()
        return DjangoWhiteNoise(django_app)

    def test_get_file_from_static_dir(self):
        url = settings.STATIC_URL + self.static_files.css_path
        response = self.server.get(url)
        self.assertEqual(response.content, self.static_files.css_content)

    def test_non_ascii_requests_safely_ignored(self):
        response = self.server.get(u"/\u263A")
        self.assertEqual(404, response.status_code)

    def test_requests_for_directory_safely_ignored(self):
        url = settings.STATIC_URL + 'directory'
        response = self.server.get(url)
        self.assertEqual(404, response.status_code)


class DjangoMiddlewareTest(DjangoWhiteNoiseTest):

    @classmethod
    def init_application(cls):
        if django.VERSION >= (1, 10):
            setting_name = 'MIDDLEWARE'
        else:
            setting_name = 'MIDDLEWARE_CLASSES'
        middleware = list(getattr(settings, setting_name) or [])
        middleware.insert(0, 'whitenoise.middleware.WhiteNoiseMiddleware')
        setattr(settings, setting_name, middleware)
        return get_wsgi_application()



import errno
import os
import stat
import sys


if sys.version_info[0] >= 3:
    BINARY_TYPE = bytes
else:
    BINARY_TYPE = str


class NotARegularFileError(Exception):
    pass


class MissingFileError(NotARegularFileError):
    pass


class IsDirectoryError(MissingFileError):
    pass


def decode_if_byte_string(s):
    if isinstance(s, BINARY_TYPE):
        s = s.decode('utf-8')
    return s


# Follow Django in treating URLs as UTF-8 encoded (which requires undoing the
# implicit ISO-8859-1 decoding applied in Python 3). Strictly speaking, URLs
# should only be ASCII anyway, but UTF-8 can be found in the wild.
if sys.version_info[0] >= 3:
    def decode_path_info(path_info):
        return path_info.encode('iso-8859-1', 'replace').decode('utf-8', 'replace')
else:
    def decode_path_info(path_info):
        return path_info.decode('utf-8', 'replace')


def stat_regular_file(path):
    """
    Wrap os.stat to raise appropriate errors if `path` is not a regular file
    """
    try:
        file_stat = os.stat(path)
    except OSError as e:
        if e.errno in (errno.ENOENT, errno.ENAMETOOLONG):
            raise MissingFileError(path)
        else:
            raise
    if not stat.S_ISREG(file_stat.st_mode):
        if stat.S_ISDIR(file_stat.st_mode):
            raise IsDirectoryError(u'Path is a directory: {0}'.format(path))
        else:
            raise NotARegularFileError(u'Not a regular file: {0}'.format(path))
    return file_stat


def ensure_leading_trailing_slash(path):
    path = (path or u'').strip(u'/')
    return u'/{0}/'.format(path) if path else u'/'



SECRET_KEY = '4'
STATIC_URL = '/static/'

ROOT_URLCONF = 'tests.django_urls'

INSTALLED_APPS = (
    'django.contrib.staticfiles',
    'whitenoise',
)

ALLOWED_HOSTS = ['*']

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'filters': {
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse'
        }
    },
    'handlers': {
        'log_to_stderr': {
            'level': 'ERROR',
            'class': 'logging.StreamHandler'
        }
    },
    'loggers': {
        'django.request': {
            'handlers': ['log_to_stderr'],
            'level': 'ERROR',
            'propagate': True,
        },
    }
}



import os
import tempfile
from unittest import TestCase
import shutil
from wsgiref.simple_server import demo_app

from .utils import TestServer, Files

from whitenoise import WhiteNoise


# Update Py2 TestCase to support Py3 method names
if not hasattr(TestCase, 'assertRegex'):
    class Py3TestCase(TestCase):
        def assertRegex(self, *args, **kwargs):
            return self.assertRegexpMatches(*args, **kwargs)
    TestCase = Py3TestCase


class WhiteNoiseTest(TestCase):

    @classmethod
    def setUpClass(cls):
        cls.files = cls.init_files()
        cls.application = cls.init_application(root=cls.files.directory)
        cls.server = TestServer(cls.application)
        super(WhiteNoiseTest, cls).setUpClass()

    @staticmethod
    def init_files():
        return Files('assets',
                     js='subdir/javascript.js',
                     gzip='compressed.css',
                     gzipped='compressed.css.gz',
                     custom_mime='custom-mime.foobar')

    @staticmethod
    def init_application(**kwargs):
        def custom_headers(headers, path, url):
            if url.endswith('.css'):
                headers['X-Is-Css-File'] = 'True'
        kwargs.update(max_age=1000,
                      mimetypes={'.foobar': 'application/x-foo-bar'},
                      add_headers_function=custom_headers)
        return WhiteNoise(demo_app, **kwargs)

    def test_get_file(self):
        response = self.server.get(self.files.js_url)
        self.assertEqual(response.content, self.files.js_content)
        self.assertRegex(response.headers['Content-Type'], r'application/javascript\b')
        self.assertRegex(response.headers['Content-Type'], r'.*\bcharset="utf-8"')

    def test_get_not_accept_gzip(self):
        response = self.server.get(self.files.gzip_url, headers={'Accept-Encoding': ''})
        self.assertEqual(response.content, self.files.gzip_content)
        self.assertEqual(response.headers.get('Content-Encoding', ''), '')
        self.assertEqual(response.headers['Vary'], 'Accept-Encoding')

    def test_get_accept_gzip(self):
        response = self.server.get(self.files.gzip_url)
        self.assertEqual(response.content, self.files.gzip_content)
        self.assertEqual(response.headers['Content-Encoding'], 'gzip')
        self.assertEqual(response.headers['Vary'], 'Accept-Encoding')

    def test_not_modified_exact(self):
        response = self.server.get(self.files.js_url)
        last_mod = response.headers['Last-Modified']
        response = self.server.get(self.files.js_url, headers={'If-Modified-Since': last_mod})
        self.assertEqual(response.status_code, 304)

    def test_not_modified_future(self):
        last_mod = 'Fri, 11 Apr 2100 11:47:06 GMT'
        response = self.server.get(self.files.js_url, headers={'If-Modified-Since': last_mod})
        self.assertEqual(response.status_code, 304)

    def test_modified(self):
        last_mod = 'Fri, 11 Apr 2001 11:47:06 GMT'
        response = self.server.get(self.files.js_url, headers={'If-Modified-Since': last_mod})
        self.assertEqual(response.status_code, 200)

    def test_max_age(self):
        response = self.server.get(self.files.js_url)
        self.assertEqual(response.headers['Cache-Control'], 'max-age=1000, public')

    def test_other_requests_passed_through(self):
        response = self.server.get('/not/static')
        self.assertIn('Hello world!', response.text)

    def test_non_ascii_requests_safely_ignored(self):
        response = self.server.get(u"/\u263A")
        self.assertIn('Hello world!', response.text)

    def test_add_under_prefix(self):
        prefix = '/prefix'
        self.application.add_files(self.files.directory, prefix=prefix)
        response = self.server.get(prefix + self.files.js_url)
        self.assertEqual(response.content, self.files.js_content)

    def test_response_has_allow_origin_header(self):
        response = self.server.get(self.files.js_url)
        self.assertEqual(response.headers.get('Access-Control-Allow-Origin'), '*')

    def test_response_has_correct_content_length_header(self):
        response = self.server.get(self.files.js_url)
        length = int(response.headers['Content-Length'])
        self.assertEqual(length, len(self.files.js_content))

    def test_gzip_response_has_correct_content_length_header(self):
        response = self.server.get(self.files.gzip_url)
        length = int(response.headers['Content-Length'])
        self.assertEqual(length, len(self.files.gzipped_content))

    def test_post_request_returns_405(self):
        response = self.server.request('post', self.files.js_url)
        self.assertEqual(response.status_code, 405)

    def test_head_request_has_no_body(self):
        response = self.server.request('head', self.files.js_url)
        self.assertEqual(response.status_code, 200)
        self.assertFalse(response.content)

    def test_custom_mimetype(self):
        response = self.server.get(self.files.custom_mime_url)
        self.assertRegex(response.headers['Content-Type'], r'application/x-foo-bar\b')

    def test_custom_headers(self):
        response = self.server.get(self.files.gzip_url)
        self.assertEqual(response.headers['x-is-css-file'], 'True')


class WhiteNoiseAutorefresh(WhiteNoiseTest):

    @classmethod
    def setUpClass(cls):
        cls.files = cls.init_files()
        cls.tmp = tempfile.mkdtemp()
        cls.application = cls.init_application(root=cls.tmp, autorefresh=True)
        cls.server = TestServer(cls.application)
        # Copy in the files *after* initializing server
        copytree(cls.files.directory, cls.tmp)
        super(WhiteNoiseTest, cls).setUpClass()

    def test_no_error_on_very_long_filename(self):
        response = self.server.get('/blah' * 1000)
        self.assertNotEqual(response.status_code, 500)

    @classmethod
    def tearDownClass(cls):
        super(WhiteNoiseTest, cls).tearDownClass()
        # Remove temporary directory
        shutil.rmtree(cls.tmp)


def copytree(src, dst):
    for name in os.listdir(src):
        src_path = os.path.join(src, name)
        dst_path = os.path.join(dst, name)
        if os.path.isdir(src_path):
            shutil.copytree(src_path, dst_path)
        else:
            shutil.copy2(src_path, dst_path)



from email.utils import formatdate
import os
from posixpath import normpath
from wsgiref.headers import Headers
from wsgiref.util import FileWrapper

from .media_types import MediaTypes
from .static_file import StaticFile
from .utils import (decode_if_byte_string, decode_path_info,
                    ensure_leading_trailing_slash, MissingFileError,
                    stat_regular_file)


class WhiteNoise(object):

    # Ten years is what nginx sets a max age if you use 'expires max;'
    # so we'll follow its lead
    FOREVER = 10*365*24*60*60

    # Attributes that can be set by keyword args in the constructor
    config_attrs = ('autorefresh', 'max_age', 'allow_all_origins', 'charset',
                    'mimetypes', 'add_headers_function')
    # Re-check the filesystem on every request so that any changes are
    # automatically picked up. NOTE: For use in development only, not supported
    # in production
    autorefresh = False
    max_age = 60
    # Set 'Access-Control-Allow-Orign: *' header on all files.
    # As these are all public static files this is safe (See
    # http://www.w3.org/TR/cors/#security) and ensures that things (e.g
    # webfonts in Firefox) still work as expected when your static files are
    # served from a CDN, rather than your primary domain.
    allow_all_origins = True
    charset = 'utf-8'
    # Custom mime types
    mimetypes = None
    # Callback for adding custom logic when setting headers
    add_headers_function = None

    def __init__(self, application, root=None, prefix=None, **kwargs):
        for attr in self.config_attrs:
            try:
                value = kwargs.pop(attr)
            except KeyError:
                pass
            else:
                value = decode_if_byte_string(value)
                setattr(self, attr, value)
        if kwargs:
            raise TypeError("Unexpected keyword argument '{0}'".format(
                list(kwargs.keys())[0]))
        self.media_types = MediaTypes(extra_types=self.mimetypes)
        self.application = application
        self.files = {}
        self.directories = []
        if root is not None:
            self.add_files(root, prefix)

    def __call__(self, environ, start_response):
        path = decode_path_info(environ['PATH_INFO'])
        if self.autorefresh:
            static_file = self.find_file(path)
        else:
            static_file = self.files.get(path)
        if static_file is None:
            return self.application(environ, start_response)
        else:
            return self.serve(static_file, environ, start_response)

    def serve(self, static_file, environ, start_response):
        response = static_file.get_response(environ['REQUEST_METHOD'], environ)
        status_line = '{} {}'.format(response.status, response.status.phrase)
        start_response(status_line, list(response.headers))
        if response.file is not None:
            file_wrapper = environ.get('wsgi.file_wrapper', FileWrapper)
            return file_wrapper(response.file)
        else:
            return []

    def add_files(self, root, prefix=None):
        root = decode_if_byte_string(root)
        prefix = decode_if_byte_string(prefix)
        prefix = ensure_leading_trailing_slash(prefix)
        if self.autorefresh:
            # Later calls to `add_files` overwrite earlier ones, hence we need
            # to store the list of directories in reverse order so later ones
            # match first when they're checked in "autorefresh" mode
            self.directories.insert(0, (root, prefix))
        else:
            self.update_files_dictionary(root, prefix)

    def update_files_dictionary(self, root, prefix):
        for directory, _, filenames in os.walk(root, followlinks=True):
            for filename in filenames:
                path = os.path.join(directory, filename)
                url = prefix + os.path.relpath(path, root).replace('\\', '/')
                self.files[url] = self.get_static_file(path, url)

    def find_file(self, url):
        # Don't bother checking URLs which could only ever be directories
        if not url or url[-1] == '/':
            return
        # Attempt to mitigate path traversal attacks. Not sure if this is
        # sufficient, hence the warning that "autorefresh" is a development
        # only feature and not for production use
        if normpath(url) != url:
            return
        for root, prefix in self.directories:
            if url.startswith(prefix):
                path = os.path.join(root, url[len(prefix):])
                try:
                    return self.get_static_file(path, url)
                except MissingFileError:
                    pass

    def get_static_file(self, path, url):
        headers = Headers([])
        self.add_stat_headers(headers, path, url)
        self.add_mime_headers(headers, path, url)
        self.add_cache_headers(headers, path, url)
        self.add_cors_headers(headers, path, url)
        self.add_extra_headers(headers, path, url)
        if self.add_headers_function:
            self.add_headers_function(headers, path, url)
        return StaticFile(path, headers)

    def add_stat_headers(self, headers, path, url):
        file_stat = stat_regular_file(path)
        headers['Last-Modified'] = formatdate(file_stat.st_mtime, usegmt=True)
        headers['Content-Length'] = str(file_stat.st_size)

    def add_mime_headers(self, headers, path, url):
        media_type = self.media_types.get_type(path)
        charset = self.get_charset(media_type, path, url)
        params = {'charset': str(charset)} if charset else {}
        headers.add_header('Content-Type', str(media_type), **params)

    def get_charset(self, media_type, path, url):
        if (media_type.startswith('text/') or
                media_type == 'application/javascript'):
            return self.charset

    def add_cache_headers(self, headers, path, url):
        if self.is_immutable_file(path, url):
            headers['Cache-Control'] = \
                    'max-age={0}, public, immutable'.format(self.FOREVER)
        elif self.max_age is not None:
            headers['Cache-Control'] = \
                    'max-age={0}, public'.format(self.max_age)

    def is_immutable_file(self, path, url):
        """
        This should be implemented by sub-classes (see e.g. DjangoWhiteNoise)
        """
        return False

    def add_cors_headers(self, headers, path, url):
        if self.allow_all_origins:
            headers['Access-Control-Allow-Origin'] = '*'

    def add_extra_headers(self, headers, path, url):
        """
        This is provided as a hook for sub-classes, by default a no-op
        """
        pass



import ast
import codecs
import os
import re
from setuptools import setup, find_packages


PROJECT_ROOT = os.path.abspath(os.path.dirname(__file__))
VERSION_RE = re.compile(r'__version__\s+=\s+(.*)')


def read(*path):
    full_path = os.path.join(PROJECT_ROOT, *path)
    with codecs.open(full_path, 'r', encoding='utf-8') as f:
        return f.read()


version_string = VERSION_RE.search(read('whitenoise/__init__.py')).group(1)
version = str(ast.literal_eval(version_string))


setup(
    name='whitenoise',
    version=version,
    author='David Evans',
    author_email='d@evans.io',
    url='http://whitenoise.evans.io',
    packages=find_packages(exclude=['tests*']),
    license='MIT',
    description="Radically simplified static file serving for WSGI applications",
    long_description=read('README.rst'),
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Topic :: Internet :: WWW/HTTP :: WSGI :: Middleware',
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.9',
        'Framework :: Django :: 1.10',
        # Temporarily removed pending resolution of
        # https://github.com/pypa/warehouse/issues/1673
        # 'Framework :: Django :: 1.11',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
    ],
)



from __future__ import unicode_literals

import contextlib
import errno
import gzip
import os
import shutil
import tempfile
from unittest import TestCase

from whitenoise.compress import main as compress_main


COMPRESSABLE_FILE = 'application.css'
TOO_SMALL_FILE = 'too-small.css'
WRONG_EXTENSION = 'image.jpg'
TEST_FILES = {
    COMPRESSABLE_FILE: b'a' * 1000,
    TOO_SMALL_FILE: b'hi',
}


class CompressTestBase(TestCase):

    @classmethod
    def setUpClass(cls):
        # Make a temporary directory and copy in test files
        cls.tmp = tempfile.mkdtemp()
        for path, contents in TEST_FILES.items():
            path = os.path.join(cls.tmp, path.lstrip('/'))
            try:
                os.makedirs(os.path.dirname(path))
            except OSError as e:
                if e.errno != errno.EEXIST:
                    raise
            with open(path, 'wb') as f:
                f.write(contents)
        cls.run_compress()
        super(CompressTestBase, cls).setUpClass()

    @classmethod
    def tearDownClass(cls):
        super(CompressTestBase, cls).tearDownClass()
        # Remove temporary directory
        shutil.rmtree(cls.tmp)


class CompressTest(CompressTestBase):

    @classmethod
    def run_compress(cls):
        compress_main(cls.tmp, quiet=True)

    def test_compresses_file(self):
        with contextlib.closing(
                gzip.open(
                    os.path.join(self.tmp, COMPRESSABLE_FILE + '.gz'), 'rb')) as f:
            contents = f.read()
        self.assertEqual(TEST_FILES[COMPRESSABLE_FILE], contents)

    def test_doesnt_compress_if_no_saving(self):
        self.assertFalse(os.path.exists(os.path.join(self.tmp, TOO_SMALL_FILE + 'gz')))

    def test_ignores_other_extensions(self):
        self.assertFalse(os.path.exists(os.path.join(self.tmp, WRONG_EXTENSION + '.gz')))



from __future__ import print_function, division, unicode_literals

import gzip
import os
import re
try:
    from io import BytesIO
except ImportError:
    from cStringIO import StringIO as BytesIO

try:
    import brotli
    brotli_installed = True
except ImportError:
    brotli_installed = False


class Compressor(object):

    # Extensions that it's not worth trying to compress
    SKIP_COMPRESS_EXTENSIONS = (
        # Images
        'jpg', 'jpeg', 'png', 'gif', 'webp',
        # Compressed files
        'zip', 'gz', 'tgz', 'bz2', 'tbz',
        # Flash
        'swf', 'flv',
        # Fonts
        'woff', 'woff2')

    def __init__(self, extensions=None, use_gzip=True, use_brotli=True,
                 log=print, quiet=False):
        if extensions is None:
            extensions = self.SKIP_COMPRESS_EXTENSIONS
        self.extension_re = self.get_extension_re(extensions)
        self.use_gzip = use_gzip
        self.use_brotli = use_brotli and brotli_installed
        if not quiet:
            self.log = log

    @staticmethod
    def get_extension_re(extensions):
        if not extensions:
            return re.compile('^$')
        else:
            return re.compile(
                r'\.({0})$'.format('|'.join(map(re.escape, extensions))),
                re.IGNORECASE)

    def should_compress(self, filename):
        return not self.extension_re.search(filename)

    def log(self, message):
        pass

    def compress(self, path):
        with open(path, 'rb') as f:
            data = f.read()
        size = len(data)
        if self.use_brotli:
            compressed = self.compress_brotli(data)
            success = self.write_data(compressed, size, path, '.br', 'Brotli')
            # If Brotli compression wasn't effective gzip won't be either
            if not success:
                return
        if self.use_gzip:
            compressed = self.compress_gzip(data)
            self.write_data(compressed, size, path, '.gz', 'Gzip')

    @staticmethod
    def compress_gzip(data):
        output = BytesIO()
        # Explicitly set mtime to 0 so gzip content is fully determined
        # by file content (0 = "no timestamp" according to gzip spec)
        with gzip.GzipFile(filename='', mode='wb', fileobj=output,
                           compresslevel=9, mtime=0) as gz_file:
            gz_file.write(data)
        return output.getvalue()

    @staticmethod
    def compress_brotli(data):
        return brotli.compress(data)

    def write_data(self, data, orig_size, path, suffix, compression):
        compressed_size = len(data)
        if not self.compressed_effectively(orig_size, compressed_size):
            self.log('Skipping {0} ({1} compression not effective)'.format(
                     path, compression))
            return False
        else:
            self.log('{0} compressed {1} ({2}K -> {3}K)'.format(
                    compression, path, orig_size // 1024,
                    compressed_size // 1024))
            with open(path + suffix, 'wb') as f:
                f.write(data)
            return True

    @staticmethod
    def compressed_effectively(orig_size, compressed_size):
        if orig_size == 0:
            return False
        ratio = compressed_size / orig_size
        return ratio <= 0.95


def main(root, **kwargs):
    compressor = Compressor(**kwargs)
    for dirpath, dirs, files in os.walk(root):
        for filename in files:
            if compressor.should_compress(filename):
                path = os.path.join(dirpath, filename)
                compressor.compress(path)


if __name__ == '__main__':
    import argparse
    parser = argparse.ArgumentParser(
            description="Search for all files inside <root> *not* matching "
                        "<extensions> and produce compressed versions with "
                        "'.gz' and '.br' suffixes (as long as this results in a "
                        "smaller file)")
    parser.add_argument('-q', '--quiet', help="Don't produce log output",
                        action='store_true')
    parser.add_argument('--no-gzip', help="Don't produce gzip '.gz' files",
                        action='store_false', dest='use_gzip')
    parser.add_argument('--no-brotli', help="Don't produce brotli '.br' files",
                        action='store_false', dest='use_brotli')
    parser.add_argument('root', help='Path root from which to search for files')
    parser.add_argument('extensions',
                        nargs='*',
                        help='File extensions to exclude from compression '
                             '(default: {})'.format(', '.join(
                                 Compressor.SKIP_COMPRESS_EXTENSIONS)),
                        default=Compressor.SKIP_COMPRESS_EXTENSIONS)
    args = parser.parse_args()
    main(**vars(args))
